@@ -34,18 +34,32 @@ public:
 	virtual std::string signature() const throw ();
 };
 
+class DHCPScan : public Scan
+{
+public:
+	DHCPScan(const std::string& name) throw ()
+		: Scan(name) {}
+
+	virtual std::string signature() const throw ();
+};
+
 class PeerScan : public Scan
 {
 protected:
 	struct ether_addr _mac;
 	IPAddress _ip;
+	IPAddress _source;
 
 public:
 	PeerScan(const std::string& name, const ether_addr& mac, const IPAddress& ip) throw ()
-		: Scan(name), _mac(mac), _ip(ip) {}
+		: Scan(name), _mac(mac), _ip(ip), _source("0.0.0.0") {}
+
+	PeerScan(const std::string& name, const ether_addr& mac, const IPAddress& ip, const IPAddress& source) throw ()
+		: Scan(name), _mac(mac), _ip(ip), _source(source) {}
 
 	const ether_addr& mac() const throw () { return _mac; }
 	const IPAddress& ip() const throw () { return _ip; }
+	const IPAddress& source() const throw () { return _source; }
 
 	virtual std::string signature() const throw ();
 };
