@@ -1,5 +1,5 @@
 /*
- * Guess the current network location using a wide selection of scans
+ * Guess the current network location
  *
  * Copyright (C) 2003  Enrico Zini <enrico@debian.org>
  * Mostly rewritten by Enrico Zini on May 2003
@@ -32,6 +32,7 @@
 
 #include "PeerScanner.h"
 #include "ScriptScanner.h"
+#include "DHCPScanner.h"
 #include "Mutex.h"
 #include "GuessnetEnvironment.h"
 #include "ScanBag.h"
@@ -73,17 +74,18 @@ protected:
 
 public:
 	ScanRunner() throw () : cand_count(0) {}
+	virtual ~ScanRunner() throw () {}
 		
 	/// Get the number of candidate profiles
 	int candidateCount() const throw () { return cand_count; }
 	
-	/// Add `scan' to the list of scans to perform
+	/// Add `scan' to the list of tests to perform
 	virtual void handleScan(const Scan* scan) throw ()
 	{
 		scans.add(scan);
 	}
 
-	/// Notify that the scan `scan' has been successful
+	/// Notify that the test `scan' has been successful
 	virtual void notifyScan(const Scan* scan) throw ()
 	{
 		try {
@@ -96,7 +98,7 @@ public:
 		}
 	}
 
-	/// Wait for an answer from the scanners for no more than `timeout' milliseconds
+	/// Wait for an answer from the tests for no more than `timeout' milliseconds
 	string getResult(int timeout) throw ()
 	{
 		struct timeval before;
@@ -137,6 +139,7 @@ protected:
 	virtual void* main() throw ()
 	{
 		interact();
+		return 0;
 	}
 	
 public:
@@ -158,16 +161,10 @@ public:
 			const Scan* scan = *i;
 			if (const PeerScan* s = dynamic_cast<const PeerScan*>(scan))
 			{
-				debug("Will scan network %.*s for %.*s (mac %.*s)\n",
+				debug("Will check network %.*s for IP address %.*s (MAC %.*s)\n",
 						PFSTR(s->name()), PFSTR(fmt(s->ip())), PFSTR(fmt(s->mac())));
 				cand_count++;
 			}
-			else if (const ScriptScan* s = dynamic_cast<const ScriptScan*>(scan))
-			{
-				debug("Will use script %.*s to test %.*s\n",
-						PFSTR(s->cmdline()), PFSTR(s->name()));
-				cand_count++;
-			}
 			else if (const LinkBeatScan* s = dynamic_cast<const LinkBeatScan*>(scan))
 			{
 				//warning("Link-beat detection currently disabled\n");
@@ -175,15 +172,27 @@ public:
 						PFSTR(s->name()));
 				cand_count++;
 			}
+			else if (const ScriptScan* s = dynamic_cast<const ScriptScan*>(scan))
+			{
+				debug("Will use command '%.*s' to test %.*s\n",
+						PFSTR(s->cmdline()), PFSTR(s->name()));
+				cand_count++;
+			}
+			if (const DHCPScan* s = dynamic_cast<const DHCPScan*>(scan))
+			{
+				debug("Will check network %.*s for DHCP service\n",
+						PFSTR(s->name()));
+				cand_count++;
+			}
 			else if (const DefaultScan* s = dynamic_cast<const DefaultScan*>(scan))
 			{
-				debug("Default scan is %.*s\n",
+				debug("Default test is %.*s\n",
 						PFSTR(s->name()));
 				cand_count++;
 			}
 			else
-				//printf("Unknown scan %p\n", scan);
-				printf("Unknown scan %.*s: %.*s\n", PFSTR(scan->name()), PFSTR(scan->signature()));
+				//printf("Unknown test %p\n", scan);
+				printf("Unknown test %.*s: %.*s\n", PFSTR(scan->name()), PFSTR(scan->signature()));
 		}
 
 		start();
@@ -205,7 +214,7 @@ public:
 			}
 			printf(" 0: quit\n");
 			printf("> ");
-			int res;
+			unsigned int res;
 			fscanf(in, "%d", &res);
 			if (res == 0)
 				canceled(true);
@@ -213,7 +222,7 @@ public:
 			if (res > 0 && res < cands.size())
 			{
 				list<const Scan*>::const_iterator i = cands.begin();
-				for (int j = 0; j < res; j++)
+				for (unsigned int j = 0; j < res; j++)
 					i++;
 				notifyScan(*i);
 			}
@@ -232,16 +241,24 @@ protected:
 
 	PeerScanner* peerScanner;
 	ScriptScanner* scriptScanner;
+	DHCPScanner* dhcpScanner;
 
 	IFace& iface;
 
+	void startNetScanning() throw (SystemException, libnetException, pcapException)
+	{
+		if (!sender)
+			sender = new NetSender(GuessnetEnvironment::get().iface());
+		if (!watcher)
+			watcher = new NetWatcher(GuessnetEnvironment::get().iface());
+	}
+
 	void startPeerScanner() throw (SystemException, libnetException, pcapException)
 	{
 		if (peerScanner)
 			return;
-		verbose("Starting network scan subsystem\n");
-		sender = new NetSender(GuessnetEnvironment::get().iface());
-		watcher = new NetWatcher(GuessnetEnvironment::get().iface());
+		verbose("Starting test subsystem\n");
+		startNetScanning();
 		peerScanner = new PeerScanner(*sender, this);
 		watcher->addARPListener(peerScanner);
 	}
@@ -250,15 +267,25 @@ protected:
 	{
 		if (scriptScanner)
 			return;
-		verbose("Starting external processes scan subsystem\n");
+		verbose("Starting external process test subsystem\n");
 		runner = new ProcessRunner();
 		scriptScanner = new ScriptScanner(*runner, this);
 	}
 
+	void startDHCPScanner() throw (SystemException, libnetException, pcapException)
+	{
+		if (dhcpScanner)
+			return;
+		verbose("Starting test subsystem\n");
+		startNetScanning();
+		dhcpScanner = new DHCPScanner(*sender, this);
+		watcher->addDHCPListener(dhcpScanner);
+	}
+
 
 public:
 	MainScanner(IFace& iface) throw (Exception) : ScanRunner(),
-		sender(0), watcher(0), runner(0), peerScanner(0), scriptScanner(0),
+		sender(0), watcher(0), runner(0), peerScanner(0), dhcpScanner(0), scriptScanner(0),
 		iface(iface) {}
 
 	void shutdown() throw (Exception)
@@ -277,7 +304,7 @@ public:
 			const Scan* scan = *i;
 			if (const PeerScan* s = dynamic_cast<const PeerScan*>(scan))
 			{
-				debug("Will scan network %.*s for %.*s (%.*s)\n",
+				debug("Will check network %.*s for IP address %.*s (MAC %.*s)\n",
 						PFSTR(s->name()), PFSTR(fmt(s->ip())), PFSTR(fmt(s->mac())));
 
 				startPeerScanner();
@@ -285,16 +312,6 @@ public:
 
 				cand_count++;
 			}
-			else if (const ScriptScan* s = dynamic_cast<const ScriptScan*>(scan))
-			{
-				debug("Will use script %.*s to test %.*s\n",
-						PFSTR(s->cmdline()), PFSTR(s->name()));
-
-				startScriptScanner();
-				scriptScanner->addCandidate(s);
-
-				cand_count++;
-			}
 			else if (const LinkBeatScan* s = dynamic_cast<const LinkBeatScan*>(scan))
 			{
 				//warning("Link-beat detection currently disabled\n");
@@ -303,7 +320,7 @@ public:
 
 				iface.update();
 				if (!iface.has_mii())
-					warning("Link beat scan requested on an interface without link beat detection support\n");
+					warning("Link beat test requested on an interface without link beat detection support\n");
 				if (!iface.connected())
 				{
 					verbose("Link beat not detected\n");
@@ -312,15 +329,35 @@ public:
 
 				cand_count++;
 			}
+			else if (const ScriptScan* s = dynamic_cast<const ScriptScan*>(scan))
+			{
+				debug("Will use command '%.*s' to test %.*s\n",
+						PFSTR(s->cmdline()), PFSTR(s->name()));
+
+				startScriptScanner();
+				scriptScanner->addCandidate(s);
+
+				cand_count++;
+			}
+			else if (const DHCPScan* s = dynamic_cast<const DHCPScan*>(scan))
+			{
+				debug("Will check network %.*s for DHCP service\n",
+						PFSTR(s->name()));
+
+				startDHCPScanner();
+				dhcpScanner->addCandidate(s);
+
+				cand_count++;
+			}
 			else if (const DefaultScan* s = dynamic_cast<const DefaultScan*>(scan))
 			{
-				debug("Default scan is %.*s\n",
+				debug("Default test is %.*s\n",
 						PFSTR(s->name()));
 				cand_count++;
 			}
 			else
-				//printf("Unknown scan %p\n", scan);
-				printf("Unknown scan %.*s: %.*s\n", PFSTR(scan->name()), PFSTR(scan->signature()));
+				//printf("Unknown test %p\n", scan);
+				printf("Unknown test %.*s: %.*s\n", PFSTR(scan->name()), PFSTR(scan->signature()));
 		}
 	}
 };
@@ -346,10 +383,15 @@ bool detectIfupdown()
 
 int main (int argc, const char *argv[])
 {
+	// Install the handler for unexpected exceptions
+	InstallUnexpected installUnexpected;
+
 	// Access the interface
 	try {
 		GuessnetEnvironment::init(argc, argv);
 
+		debug("Guessnet " VERSION " starting...\n");
+
 		IFace iface(GuessnetEnvironment::get().iface());
 
 		// After the interface is up, we need another try run to be able to
@@ -357,9 +399,6 @@ int main (int argc, const char *argv[])
 		bool iface_was_down;
 		if_params saved_iface_cfg;
 		try {
-			// Install the handler for unexpected exceptions
-			InstallUnexpected installUnexpected;
-
 			vector<const Scan*> scans = GuessnetEnvironment::get().scans();
 			verbose("%d candidate profiles\n", scans.size());
 
@@ -369,7 +408,7 @@ int main (int argc, const char *argv[])
 			iface_was_down = !iface.up();
 			if (iface_was_down)
 			{
-				verbose("Interface %.*s was down: initializing for broadcast", PFSTR(iface.name()));
+				verbose("Interface %.*s was down: initializing for broadcast\n", PFSTR(iface.name()));
 				saved_iface_cfg = iface.initBroadcast(GuessnetEnvironment::get().initTimeout());
 			}
 
@@ -393,28 +432,28 @@ int main (int argc, const char *argv[])
 			// Scanning methods
 			MainScanner scanner(iface);
 			//FakeScanner scanner;
-			debug("Initialized scanning subsystems\n");
+			debug("Initialized test subsystems\n");
 
 			scanner.handleScan(new DefaultScan(GuessnetEnvironment::get().defprof()));
-			debug("Added \"default\" scan %.*s\n", PFSTR(GuessnetEnvironment::get().defprof()));
+			debug("Added \"default\" test %.*s\n", PFSTR(GuessnetEnvironment::get().defprof()));
 
 			for (vector<const Scan*>::const_iterator i = scans.begin(); i != scans.end(); i++)
 				scanner.handleScan(*i);
 			
 			scanner.startScans();
-			debug("Started scans\n");
+			debug("Started tests\n");
 
-			// Wait for the first result from the scans
+			// Wait for the first result from the tests
 			string profile;
 			if (scanner.candidateCount() > 1)
 			{
 				debug("%d candidates\n", scanner.candidateCount());
 				profile = scanner.getResult(GuessnetEnvironment::get().timeout() * 1000);
 			} else {
-				warning("no candidates provided: skipping detection");
+				warning("No candidates provided: skipping detection\n");
 			}
 
-			// Shutdown the scanners
+			// Shutdown the tests
 			scanner.shutdown();
 
 			// We've shutdown the threads: restore original signals
